@@ -1,7 +1,8 @@
+/* $Id: buffers.c,v 1.9.4.3 2000/11/05 21:24:00 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
+ * Version:  3.4
  * 
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  * 
@@ -167,7 +168,7 @@ clear_color_buffer(GLcontext *ctx)
    }
    else {
       /* Color index mode */
-      ASSERT(ctx->Color.IndexMask == ~0);
+      ASSERT(ctx->Color.IndexMask == (GLuint) ~0);
       if (ctx->Visual->IndexBits == 8) {
          /* 8-bit clear */
          GLubyte span[MAX_WIDTH];
@@ -208,15 +209,19 @@ clear_color_buffers(GLcontext *ctx)
       if (bufferBit & ctx->Color.DrawDestMask) {
          if (bufferBit == FRONT_LEFT_BIT) {
             (void) (*ctx->Driver.SetDrawBuffer)( ctx, GL_FRONT_LEFT);
+            (void) (*ctx->Driver.SetReadBuffer)( ctx, ctx->DrawBuffer, GL_FRONT_LEFT);
          }
          else if (bufferBit == FRONT_RIGHT_BIT) {
             (void) (*ctx->Driver.SetDrawBuffer)( ctx, GL_FRONT_RIGHT);
+            (void) (*ctx->Driver.SetReadBuffer)( ctx, ctx->DrawBuffer, GL_FRONT_RIGHT);
          }
          else if (bufferBit == BACK_LEFT_BIT) {
             (void) (*ctx->Driver.SetDrawBuffer)( ctx, GL_BACK_LEFT);
+            (void) (*ctx->Driver.SetReadBuffer)( ctx, ctx->DrawBuffer, GL_BACK_LEFT);
          }
          else {
             (void) (*ctx->Driver.SetDrawBuffer)( ctx, GL_BACK_RIGHT);
+            (void) (*ctx->Driver.SetReadBuffer)( ctx, ctx->DrawBuffer, GL_BACK_RIGHT);
          }
          
          if (ctx->Color.SWmasking) {
@@ -228,8 +233,9 @@ clear_color_buffers(GLcontext *ctx)
       }
    }
 
-   /* restore default dest buffer */
+   /* restore default read/draw buffers */
    (void) (*ctx->Driver.SetDrawBuffer)( ctx, ctx->Color.DriverDrawBuffer );
+   (void) (*ctx->Driver.SetReadBuffer)( ctx, ctx->ReadBuffer, ctx->Pixel.DriverReadBuffer );
 }
 
 
@@ -287,6 +293,8 @@ _mesa_Clear( GLbitfield mask )
       }
 #endif
 
+      RENDER_START(ctx);
+
       /* do software clearing here */
       if (newMask) {
          if (newMask & ctx->Color.DrawDestMask)   clear_color_buffers(ctx);
@@ -302,6 +310,8 @@ _mesa_Clear( GLbitfield mask )
          _mesa_clear_alpha_buffers( ctx );
       }
 
+      RENDER_FINISH(ctx);
+
 #ifdef PROFILE
       ctx->ClearTime += gl_time() - t0;
       ctx->ClearCount++;
