@@ -1,62 +1,49 @@
-/* $Id: hash.c,v 1.4 1998/02/07 14:25:12 brianp Exp $ */
+/* $Id: hash.c,v 1.9.4.1 2000/11/26 21:10:26 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-
-/*
- * $Log: hash.c,v $
- * Revision 1.4  1998/02/07 14:25:12  brianp
- * fixed small Sun compiler warning (John Stone)
- *
- * Revision 1.3  1997/09/22 02:33:07  brianp
- * added HashRemove() and HashFirstEntry()
- *
- * Revision 1.2  1997/09/03 13:13:45  brianp
- * added a few pointer casts
- *
- * Revision 1.1  1997/08/22 01:15:10  brianp
- * Initial revision
- *
+ * Version:  3.3
+ * 
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
 #ifdef PC_HEADER
 #include "all.h"
 #else
-#include <assert.h>
-#include <stdlib.h>
-#include <stdio.h>
+#include "glheader.h"
+#include "glthread.h"
 #include "hash.h"
+#include "mem.h"
 #endif
 
 
 /*
- * Generic hash table.  Only dependency is the GLuint datatype.
+ * Generic hash table.
  *
  * This is used to implement display list and texture object lookup.
  * NOTE: key=0 is illegal.
  */
 
 
-#define TABLE_SIZE 1001
+#define TABLE_SIZE 1024
 
 struct HashEntry {
    GLuint Key;
@@ -64,9 +51,10 @@ struct HashEntry {
    struct HashEntry *Next;
 };
 
-struct HashTable {
+struct _mesa_HashTable {
    struct HashEntry *Table[TABLE_SIZE];
    GLuint MaxKey;
+   _glthread_Mutex Mutex;
 };
 
 
@@ -74,9 +62,9 @@ struct HashTable {
 /*
  * Return pointer to a new, empty hash table.
  */
-struct HashTable *NewHashTable(void)
+struct _mesa_HashTable *_mesa_NewHashTable(void)
 {
-   return (struct HashTable *) calloc(sizeof (struct HashTable), 1);
+   return CALLOC_STRUCT(_mesa_HashTable);
 }
 
 
@@ -84,7 +72,7 @@ struct HashTable *NewHashTable(void)
 /*
  * Delete a hash table.
  */
-void DeleteHashTable(struct HashTable *table)
+void _mesa_DeleteHashTable(struct _mesa_HashTable *table)
 {
    GLuint i;
    assert(table);
@@ -92,11 +80,11 @@ void DeleteHashTable(struct HashTable *table)
       struct HashEntry *entry = table->Table[i];
       while (entry) {
 	 struct HashEntry *next = entry->Next;
-	 free(entry);
+	 FREE(entry);
 	 entry = next;
       }
    }
-   free(table);
+   FREE(table);
 }
 
 
@@ -107,15 +95,14 @@ void DeleteHashTable(struct HashTable *table)
  *         key - the key
  * Return:  user data pointer or NULL if key not in table
  */
-void *HashLookup(const struct HashTable *table, GLuint key)
+void *_mesa_HashLookup(const struct _mesa_HashTable *table, GLuint key)
 {
    GLuint pos;
-   struct HashEntry *entry;
+   const struct HashEntry *entry;
 
    assert(table);
-   assert(key);
 
-   pos = key % TABLE_SIZE;
+   pos = key & (TABLE_SIZE-1);
    entry = table->Table[pos];
    while (entry) {
       if (entry->Key == key) {
@@ -135,35 +122,39 @@ void *HashLookup(const struct HashTable *table, GLuint key)
  *         key - the key (not zero)
  *         data - pointer to user data
  */
-void HashInsert(struct HashTable *table, GLuint key, void *data)
+void _mesa_HashInsert(struct _mesa_HashTable *table, GLuint key, void *data)
 {
    /* search for existing entry with this key */
    GLuint pos;
    struct HashEntry *entry;
 
    assert(table);
-   assert(key);
+
+   _glthread_LOCK_MUTEX(table->Mutex);
 
    if (key > table->MaxKey)
       table->MaxKey = key;
 
-   pos = key % TABLE_SIZE;
+   pos = key & (TABLE_SIZE-1);
    entry = table->Table[pos];
    while (entry) {
       if (entry->Key == key) {
          /* replace entry's data */
 	 entry->Data = data;
+         _glthread_UNLOCK_MUTEX(table->Mutex);
 	 return;
       }
       entry = entry->Next;
    }
 
    /* alloc and insert new table entry */
-   entry = (struct HashEntry *) calloc(sizeof(struct HashEntry), 1);
+   entry = MALLOC_STRUCT(HashEntry);
    entry->Key = key;
    entry->Data = data;
    entry->Next = table->Table[pos];
    table->Table[pos] = entry;
+
+   _glthread_UNLOCK_MUTEX(table->Mutex);
 }
 
 
@@ -173,15 +164,16 @@ void HashInsert(struct HashTable *table, GLuint key, void *data)
  * Input:  table - the hash table
  *         key - key of entry to remove
  */
-void HashRemove(struct HashTable *table, GLuint key)
+void _mesa_HashRemove(struct _mesa_HashTable *table, GLuint key)
 {
    GLuint pos;
    struct HashEntry *entry, *prev;
 
    assert(table);
-   assert(key);
 
-   pos = key % TABLE_SIZE;
+   _glthread_LOCK_MUTEX(table->Mutex);
+
+   pos = key & (TABLE_SIZE-1);
    prev = NULL;
    entry = table->Table[pos];
    while (entry) {
@@ -193,29 +185,36 @@ void HashRemove(struct HashTable *table, GLuint key)
          else {
             table->Table[pos] = entry->Next;
          }
-         free(entry);
+         FREE(entry);
+         _glthread_UNLOCK_MUTEX(table->Mutex);
 	 return;
       }
       prev = entry;
       entry = entry->Next;
    }
+
+   _glthread_UNLOCK_MUTEX(table->Mutex);
 }
 
 
 
 /*
  * Return the key of the "first" entry in the hash table.
- * By calling this function until zero is returned we can get
- * the keys of all entries in the table.
+ * This is used in the course of deleting all display lists when
+ * a context is destroyed.
  */
-GLuint HashFirstEntry(const struct HashTable *table)
+GLuint _mesa_HashFirstEntry(struct _mesa_HashTable *table)
 {
    GLuint pos;
    assert(table);
+   _glthread_LOCK_MUTEX(table->Mutex);
    for (pos=0; pos < TABLE_SIZE; pos++) {
-      if (table->Table[pos])
+      if (table->Table[pos]) {
+         _glthread_UNLOCK_MUTEX(table->Mutex);
          return table->Table[pos]->Key;
+      }
    }
+   _glthread_UNLOCK_MUTEX(table->Mutex);
    return 0;
 }
 
@@ -224,12 +223,12 @@ GLuint HashFirstEntry(const struct HashTable *table)
 /*
  * Dump contents of hash table for debugging.
  */
-void HashPrint(const struct HashTable *table)
+void _mesa_HashPrint(const struct _mesa_HashTable *table)
 {
    GLuint i;
    assert(table);
    for (i=0;i<TABLE_SIZE;i++) {
-      struct HashEntry *entry = table->Table[i];
+      const struct HashEntry *entry = table->Table[i];
       while (entry) {
 	 printf("%u %p\n", entry->Key, entry->Data);
 	 entry = entry->Next;
@@ -243,22 +242,24 @@ void HashPrint(const struct HashTable *table)
  * Find a block of 'numKeys' adjacent unused hash keys.
  * Input:  table - the hash table
  *         numKeys - number of keys needed
- * Return:  startint key of free block or 0 if failure
+ * Return:  starting key of free block or 0 if failure
  */
-GLuint HashFindFreeKeyBlock(const struct HashTable *table, GLuint numKeys)
+GLuint _mesa_HashFindFreeKeyBlock(struct _mesa_HashTable *table, GLuint numKeys)
 {
    GLuint maxKey = ~((GLuint) 0);
+   _glthread_LOCK_MUTEX(table->Mutex);
    if (maxKey - numKeys > table->MaxKey) {
       /* the quick solution */
+      _glthread_UNLOCK_MUTEX(table->Mutex);
       return table->MaxKey + 1;
    }
    else {
       /* the slow solution */
       GLuint freeCount = 0;
-      GLuint freeStart = 0;
+      GLuint freeStart = 1;
       GLuint key;
-      for (key=0; key!=maxKey; key++) {
-	 if (HashLookup(table, key)) {
+      for (key=1; key!=maxKey; key++) {
+	 if (_mesa_HashLookup(table, key)) {
 	    /* darn, this key is already in use */
 	    freeCount = 0;
 	    freeStart = key+1;
@@ -267,11 +268,13 @@ GLuint HashFindFreeKeyBlock(const struct HashTable *table, GLuint numKeys)
 	    /* this key not in use, check if we've found enough */
 	    freeCount++;
 	    if (freeCount == numKeys) {
+               _glthread_UNLOCK_MUTEX(table->Mutex);
 	       return freeStart;
 	    }
 	 }
       }
       /* cannot allocate a block of numKeys consecutive keys */
+      _glthread_UNLOCK_MUTEX(table->Mutex);
       return 0;
    }
 }
@@ -287,15 +290,15 @@ int main(int argc, char *argv[])
    printf("&a = %p\n", &a);
    printf("&b = %p\n", &b);
 
-   t = NewHashTable();
-   HashInsert(t, 501, &a);
-   HashInsert(t, 10, &c);
-   HashInsert(t, 0xfffffff8, &b);
-   HashPrint(t);
-   printf("Find 501: %p\n", HashLookup(t,501));
-   printf("Find 1313: %p\n", HashLookup(t,1313));
-   printf("Find block of 100: %d\n", HashFindFreeKeyBlock(t, 100));
-   DeleteHashTable(t);
+   t = _mesa_NewHashTable();
+   _mesa_HashInsert(t, 501, &a);
+   _mesa_HashInsert(t, 10, &c);
+   _mesa_HashInsert(t, 0xfffffff8, &b);
+   _mesa_HashPrint(t);
+   printf("Find 501: %p\n", _mesa_HashLookup(t,501));
+   printf("Find 1313: %p\n", _mesa_HashLookup(t,1313));
+   printf("Find block of 100: %d\n", _mesa_HashFindFreeKeyBlock(t, 100));
+   _mesa_DeleteHashTable(t);
 
    return 0;
 }
