@@ -1,8 +1,8 @@
-/* $Id: imaging.c,v 1.15 2000/06/27 15:47:59 brianp Exp $ */
+/* $Id: imaging.c,v 1.15.4.3 2000/11/21 23:00:21 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
+ * Version:  3.4
  *
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  *
@@ -211,7 +211,7 @@ pack_minmax( GLcontext *ctx, CONST GLfloat minmax[2][4],
              const struct gl_pixelstore_attrib *packing )
 {
    const GLint comps = _mesa_components_in_format(format);
-   GLuint luminance[2];
+   GLfloat luminance[2];
 
    if (format == GL_LUMINANCE || format == GL_LUMINANCE_ALPHA) {
       luminance[0] = minmax[0][RCOMP] + minmax[0][GCOMP] + minmax[0][BCOMP];
@@ -1255,7 +1255,7 @@ void
 _mesa_CopyConvolutionFilter1D(GLenum target, GLenum internalFormat, GLint x, GLint y, GLsizei width)
 {
    GLenum baseFormat;
-   GLfloat rgba[MAX_CONVOLUTION_WIDTH][4];
+   GLubyte rgba[MAX_CONVOLUTION_WIDTH][4];
    GET_CURRENT_CONTEXT(ctx);
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glCopyConvolutionFilter1D");
 
@@ -1276,7 +1276,9 @@ _mesa_CopyConvolutionFilter1D(GLenum target, GLenum internalFormat, GLint x, GLi
    }
 
    /* read pixels from framebuffer */
+   RENDER_START(ctx);
    gl_read_rgba_span(ctx, ctx->ReadBuffer, width, x, y, (GLubyte (*)[4]) rgba);
+   RENDER_FINISH(ctx);
 
    /* store as convolution filter */
    _mesa_ConvolutionFilter1D(target, internalFormat, width,
@@ -1290,7 +1292,7 @@ _mesa_CopyConvolutionFilter2D(GLenum target, GLenum internalFormat, GLint x, GLi
    GLenum baseFormat;
    GLint i;
    struct gl_pixelstore_attrib packSave;
-   GLfloat rgba[MAX_CONVOLUTION_HEIGHT][MAX_CONVOLUTION_WIDTH][4];
+   GLubyte rgba[MAX_CONVOLUTION_HEIGHT][MAX_CONVOLUTION_WIDTH][4];
    GET_CURRENT_CONTEXT(ctx);
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glCopyConvolutionFilter2D");
 
@@ -1315,10 +1317,12 @@ _mesa_CopyConvolutionFilter2D(GLenum target, GLenum internalFormat, GLint x, GLi
    }
 
    /* read pixels from framebuffer */
+   RENDER_START(ctx);
    for (i = 0; i < height; i++) {
       gl_read_rgba_span(ctx, ctx->ReadBuffer, width, x, y + i,
                         (GLubyte (*)[4]) rgba[i]);
    }
+   RENDER_FINISH(ctx);
 
    /*
     * store as convolution filter
@@ -1344,6 +1348,8 @@ _mesa_CopyConvolutionFilter2D(GLenum target, GLenum internalFormat, GLint x, GLi
 void
 _mesa_GetConvolutionFilter(GLenum target, GLenum format, GLenum type, GLvoid *image)
 {
+   const struct gl_convolution_attrib *filter;
+   GLint row;
    GET_CURRENT_CONTEXT(ctx);
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glGetConvolutionFilter");
 
@@ -1362,8 +1368,31 @@ _mesa_GetConvolutionFilter(GLenum target, GLenum format, GLenum type, GLvoid *im
       return;
    }
 
-   (void) image;
-   /* XXX store image */
+   switch (target) {
+      case GL_CONVOLUTION_1D:
+         filter = &(ctx->Convolution1D);
+         break;
+      case GL_CONVOLUTION_2D:
+         filter = &(ctx->Convolution2D);
+         break;
+      default:
+         gl_error(ctx, GL_INVALID_ENUM, "glGetConvolutionFilter(target)");
+         return;
+   }
+
+   for (row = 0; row < filter->Height; row++) {
+      GLvoid *dst = _mesa_image_address( &ctx->Pack, image, filter->Width,
+                                         filter->Height, format, type,
+                                         0, row, 0);
+      const GLfloat *src = filter->Filter + row * filter->Width * 4;
+      GLubyte img[MAX_WIDTH * 4];
+      GLint i;
+      for (i = 0; i < filter->Width * 4; i++) {
+         img[i] = (GLint) (CLAMP(src[i], 0.0, 1.0) * 255.0);
+      }
+      _mesa_pack_rgba_span(ctx, filter->Width, (const GLubyte (*)[4]) img,
+                           format, type, dst, &ctx->Pack, 0);
+   }
 }
 
 
@@ -1464,7 +1493,7 @@ _mesa_GetConvolutionParameteriv(GLenum target, GLenum pname, GLint *params)
          params[3] = FLOAT_TO_INT(ctx->Pixel.ConvolutionBorderColor[c][3]);
          break;
       case GL_CONVOLUTION_BORDER_MODE:
-         *params = (GLint) ctx->Pixel.ConvolutionBorderMode;
+         *params = (GLint) ctx->Pixel.ConvolutionBorderMode[c];
          break;
       case GL_CONVOLUTION_FILTER_SCALE:
          params[0] = (GLint) ctx->Pixel.ConvolutionFilterScale[c][0];
