@@ -1,21 +1,21 @@
-/* $Id: texture.c,v 1.22 2000/07/05 16:14:23 brianp Exp $ */
+/* $Id: texture.c,v 1.22.4.7 2001/02/12 20:39:34 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
- * 
- * Copyright (C) 1999  Brian Paul   All Rights Reserved.
- * 
+ * Version:  3.4.1
+ *
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
@@ -62,16 +62,16 @@ static texgen_func texgen_sphere_map_tab[4];
 
 typedef void (*build_m_func)(GLfloat f[][3],
 			     GLfloat m[],
-			     const GLvector3f *normals, 
-			     const GLvector4f *coord_vec, 
+			     const GLvector3f *normals,
+			     const GLvector4f *coord_vec,
 			     const GLuint flags[],
 			     const GLubyte cullmask[] );
 
 
-typedef void (*build_f_func)( GLfloat *f, 
+typedef void (*build_f_func)( GLfloat *f,
 			      GLuint fstride,
 			      const GLvector3f *normal_vec,
-			      const GLvector4f *coord_vec, 
+			      const GLvector4f *coord_vec,
 			      const GLuint flags[],
 			      const GLubyte cullmask[] );
 
@@ -80,41 +80,42 @@ typedef void (*build_f_func)( GLfloat *f,
  *     gains to texture generation, but it is still necessary to cope
  *     with the two different formats.
  */
+#define IDX              0
 #define TAG(x)           x
 #define FIRST_NORMAL     normals->start
+#define CUR_NORMAL       (void) normal
 #define NEXT_NORMAL      STRIDE_F(normal, normals->stride)
-#define LOCAL_VARS
 #define CHECK            
-#define IDX              0
+#define LOCAL_VARS
 #include "texgen_tmp.h"
 
 
+#define IDX              2
 #define TAG(x)           x##_compacted
 #define FIRST_NORMAL     normals->start
-#define NEXT_NORMAL      ((flags[i]&VERT_NORM) ? (normal=first_normal[i]) : (normal))
+#define CUR_NORMAL       ((flags[i]&VERT_NORM) ? normal=normal_ptr : normal)
+#define NEXT_NORMAL      STRIDE_F(normal_ptr, normals->stride)
 #define CHECK            
-#define IDX              2
-#define LOCAL_VARS        \
-   GLfloat (*first_normal)[3] = (GLfloat (*)[3]) FIRST_NORMAL;
-
+#define LOCAL_VARS       const GLfloat *normal_ptr = FIRST_NORMAL;
 #include "texgen_tmp.h"
 
+
+#define IDX              1
 #define TAG(x)           x##_masked
 #define FIRST_NORMAL     normals->start
+#define CUR_NORMAL       (void) normal
 #define NEXT_NORMAL      STRIDE_F(normal, normals->stride)
-#define LOCAL_VARS       
 #define CHECK            if (cullmask[i])
-#define IDX              1
+#define LOCAL_VARS       
 #include "texgen_tmp.h"
 
+#define IDX              3
 #define TAG(x)           x##_compacted_masked
 #define FIRST_NORMAL     normals->start
-#define NEXT_NORMAL      ((flags[i]&VERT_NORM) ? normal=first_normal[i] : 0)
+#define CUR_NORMAL       ((flags[i]&VERT_NORM) ? normal=normal_ptr : normal)
+#define NEXT_NORMAL      STRIDE_F(normal_ptr, normals->stride)
 #define CHECK            if (cullmask[i])
-#define IDX              3
-#define LOCAL_VARS       \
-   GLfloat (*first_normal)[3] = (GLfloat (*)[3]) FIRST_NORMAL;
-
+#define LOCAL_VARS       const GLfloat *normal_ptr = FIRST_NORMAL;
 #include "texgen_tmp.h"
 
 
@@ -190,7 +191,7 @@ void gl_update_texture_unit( GLcontext *ctx, struct gl_texture_unit *texUnit )
 	    fprintf(stderr, "Bad value for texUnit->Enabled %x\n",
 		    texUnit->Enabled);
 	    break;
-	 }	 
+	 }
       }
 
       texUnit->ReallyEnabled = 0;
@@ -220,7 +221,7 @@ void gl_update_texture_unit( GLcontext *ctx, struct gl_texture_unit *texUnit )
          sz = 4;
          texUnit->GenFlags |= texUnit->GenBitR;
       }
-   
+
       texUnit->TexgenSize = sz;
       texUnit->Holes = (GLubyte) (all_bits[sz] & ~texUnit->TexGenEnabled);
       texUnit->func = texgen_generic_tab;
@@ -233,7 +234,7 @@ void gl_update_texture_unit( GLcontext *ctx, struct gl_texture_unit *texUnit )
 	    texUnit->func = texgen_normal_map_nv_tab;
 	 }
       }
-      else if (texUnit->TexGenEnabled == (S_BIT|T_BIT) && 
+      else if (texUnit->TexGenEnabled == (S_BIT|T_BIT) &&
 	       texUnit->GenFlags == TEXGEN_SPHERE_MAP) {
 	 texUnit->func = texgen_sphere_map_tab;
       }
@@ -399,7 +400,7 @@ static void palette_sample(const struct gl_texture_object *tObj,
 }
 
 
-   
+
 
 /*
  * Bitflags for texture border color sampling.
@@ -2004,10 +2005,11 @@ _mesa_set_texture_sampler( struct gl_texture_object *t )
 
 
 #define PROD(A,B)   ( (GLuint)(A) * ((GLuint)(B)+1) )
+#define S_PROD(A,B) ( (GLint)(A) * ((GLint)(B)+1) )
 
 static INLINE void
-_mesa_texture_combine(CONST GLcontext *ctx,
-                      CONST struct gl_texture_unit *textureUnit,
+_mesa_texture_combine(const GLcontext *ctx,
+                      const struct gl_texture_unit *textureUnit,
                       GLuint n,
                       GLubyte (*primary_rgba)[4],
                       GLubyte (*texel)[4],
@@ -2036,9 +2038,9 @@ _mesa_texture_combine(CONST GLcontext *ctx,
          case GL_CONSTANT_EXT:
             {
                GLubyte (*c)[4] = ccolor[j];
-               GLubyte col = FLOAT_TO_UBYTE(textureUnit->EnvColor[3]);
+               GLubyte alpha = FLOAT_TO_UBYTE(textureUnit->EnvColor[3]);
                for (i = 0; i < n; i++)
-                  c[i][ACOMP] = col;
+                  c[i][ACOMP] = alpha;
                argA[j] = ccolor[j];
             }
             break;
@@ -2046,17 +2048,6 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             gl_problem(NULL, "invalid combine source");
       }
 
-      if (textureUnit->CombineOperandA[j] == GL_ONE_MINUS_SRC_ALPHA) {
-         GLubyte (*src)[4] = argA[j];
-         GLubyte (*dst)[4] = ccolor[j];
-         argA[j] = ccolor[j];
-         for (i = 0; i < n; i++)
-            dst[i][ACOMP] = 255 - src[i][ACOMP];
-      }
-
-      if (j == 2)        /* arg2 has no color component in specification. */
-         break;
-
       switch (textureUnit->CombineSourceRGB[j]) {
          case GL_TEXTURE:
             argRGB[j] = texel;
@@ -2070,16 +2061,13 @@ _mesa_texture_combine(CONST GLcontext *ctx,
          case GL_CONSTANT_EXT:
             {
                GLubyte (*c)[4] = ccolor[j];
-               GLubyte col[4];
-
-               col[RCOMP] = FLOAT_TO_UBYTE(textureUnit->EnvColor[0]);
-               col[GCOMP] = FLOAT_TO_UBYTE(textureUnit->EnvColor[1]);
-               col[BCOMP] = FLOAT_TO_UBYTE(textureUnit->EnvColor[2]);
-
+               const GLubyte red   = FLOAT_TO_UBYTE(textureUnit->EnvColor[0]);
+               const GLubyte green = FLOAT_TO_UBYTE(textureUnit->EnvColor[1]);
+               const GLubyte blue  = FLOAT_TO_UBYTE(textureUnit->EnvColor[2]);
                for (i = 0; i < n; i++) {
-                  c[i][RCOMP] = col[RCOMP];
-                  c[i][GCOMP] = col[GCOMP];
-                  c[i][BCOMP] = col[BCOMP];
+                  c[i][RCOMP] = red;
+                  c[i][GCOMP] = green;
+                  c[i][BCOMP] = blue;
                }
                argRGB[j] = ccolor[j];
             }
@@ -2100,13 +2088,17 @@ _mesa_texture_combine(CONST GLcontext *ctx,
                dst[i][GCOMP] = 255 - src[i][GCOMP];
                dst[i][BCOMP] = 255 - src[i][BCOMP];
             }
-         } else if (textureUnit->CombineOperandRGB[j] == GL_SRC_ALPHA) {
+         }
+         else if (textureUnit->CombineOperandRGB[j] == GL_SRC_ALPHA) {
+            src = argA[j];
             for (i = 0; i < n; i++) {
                dst[i][RCOMP] = src[i][ACOMP];
                dst[i][GCOMP] = src[i][ACOMP];
                dst[i][BCOMP] = src[i][ACOMP];
             }
-         } else {                      /*  GL_ONE_MINUS_SRC_ALPHA  */
+         }
+         else {                      /*  GL_ONE_MINUS_SRC_ALPHA  */
+            src = argA[j];
             for (i = 0; i < n; i++) {
                dst[i][RCOMP] = 255 - src[i][ACOMP];
                dst[i][GCOMP] = 255 - src[i][ACOMP];
@@ -2115,6 +2107,15 @@ _mesa_texture_combine(CONST GLcontext *ctx,
          }
       }
 
+      if (textureUnit->CombineOperandA[j] == GL_ONE_MINUS_SRC_ALPHA) {
+         GLubyte (*src)[4] = argA[j];
+         GLubyte (*dst)[4] = ccolor[j];
+         argA[j] = ccolor[j];
+         for (i = 0; i < n; i++) {
+            dst[i][ACOMP] = 255 - src[i][ACOMP];
+         }
+      }
+
       if (textureUnit->CombineModeRGB == GL_REPLACE &&
           textureUnit->CombineModeA == GL_REPLACE) {
          break;      /*  done, we need only arg0  */
@@ -2131,10 +2132,22 @@ _mesa_texture_combine(CONST GLcontext *ctx,
       case GL_REPLACE:
          {
             const GLubyte (*arg0)[4] = (const GLubyte (*)[4]) argRGB[0];
-            for (i = 0; i < n; i++) {
-               rgba[i][RCOMP] = arg0[i][RCOMP] << RGBshift;
-               rgba[i][GCOMP] = arg0[i][GCOMP] << RGBshift;
-               rgba[i][BCOMP] = arg0[i][BCOMP] << RGBshift;
+            if (RGBshift) {
+               for (i = 0; i < n; i++) {
+                  GLuint r = (GLuint) arg0[i][RCOMP] << RGBshift;
+                  GLuint g = (GLuint) arg0[i][GCOMP] << RGBshift;
+                  GLuint b = (GLuint) arg0[i][BCOMP] << RGBshift;
+                  rgba[i][RCOMP] = MIN2(r, 255);
+                  rgba[i][GCOMP] = MIN2(g, 255);
+                  rgba[i][BCOMP] = MIN2(b, 255);
+               }
+            }
+            else {
+               for (i = 0; i < n; i++) {
+                  rgba[i][RCOMP] = arg0[i][RCOMP];
+                  rgba[i][GCOMP] = arg0[i][GCOMP];
+                  rgba[i][BCOMP] = arg0[i][BCOMP];
+               }
             }
          }
          break;
@@ -2144,9 +2157,12 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             const GLubyte (*arg1)[4] = (const GLubyte (*)[4]) argRGB[1];
             RGBshift = 8 - RGBshift;
             for (i = 0; i < n; i++) {
-               rgba[i][RCOMP] = (GLubyte) (PROD(arg0[i][0], arg1[i][RCOMP]) >> RGBshift);
-               rgba[i][GCOMP] = (GLubyte) (PROD(arg0[i][1], arg1[i][GCOMP]) >> RGBshift);
-               rgba[i][BCOMP] = (GLubyte) (PROD(arg0[i][2], arg1[i][BCOMP]) >> RGBshift);
+               GLuint r = PROD(arg0[i][0], arg1[i][RCOMP]) >> RGBshift;
+               GLuint g = PROD(arg0[i][1], arg1[i][GCOMP]) >> RGBshift;
+               GLuint b = PROD(arg0[i][2], arg1[i][BCOMP]) >> RGBshift;
+               rgba[i][RCOMP] = (GLubyte) MIN2(r, 255);
+               rgba[i][GCOMP] = (GLubyte) MIN2(g, 255);
+               rgba[i][BCOMP] = (GLubyte) MIN2(b, 255);
             }
          }
          break;
@@ -2158,9 +2174,9 @@ _mesa_texture_combine(CONST GLcontext *ctx,
                GLint r = ((GLuint) arg0[i][RCOMP] + arg1[i][RCOMP]) << RGBshift;
                GLint g = ((GLuint) arg0[i][GCOMP] + arg1[i][GCOMP]) << RGBshift;
                GLint b = ((GLuint) arg0[i][BCOMP] + arg1[i][BCOMP]) << RGBshift;
-               rgba[i][RCOMP] = (r > 255) ? 255 : (GLubyte) r;
-               rgba[i][GCOMP] = (g > 255) ? 255 : (GLubyte) g;
-               rgba[i][BCOMP] = (b > 255) ? 255 : (GLubyte) b;
+               rgba[i][RCOMP] = (GLubyte) MIN2(r, 255);
+               rgba[i][GCOMP] = (GLubyte) MIN2(g, 255);
+               rgba[i][BCOMP] = (GLubyte) MIN2(b, 255);
             }
          }
          break;
@@ -2175,9 +2191,9 @@ _mesa_texture_combine(CONST GLcontext *ctx,
                r = (r < 0) ? 0 : r << RGBshift;
                b = (b < 0) ? 0 : b << RGBshift;
                g = (g < 0) ? 0 : g << RGBshift;
-               rgba[i][RCOMP] = (r > 255) ? 255 : (GLubyte) r;
-               rgba[i][GCOMP] = (g > 255) ? 255 : (GLubyte) g;
-               rgba[i][BCOMP] = (b > 255) ? 255 : (GLubyte) b;
+               rgba[i][RCOMP] = (GLubyte) MIN2(r, 255);
+               rgba[i][GCOMP] = (GLubyte) MIN2(g, 255);
+               rgba[i][BCOMP] = (GLubyte) MIN2(b, 255);
             }
          }
          break;
@@ -2188,9 +2204,41 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             const GLubyte (*arg2)[4] = (const GLubyte (*)[4]) argRGB[2];
             RGBshift = 8 - RGBshift;
             for (i = 0; i < n; i++) {
-               rgba[i][RCOMP] = (GLubyte) ((PROD(arg0[i][RCOMP], arg2[i][ACOMP]) + PROD(arg1[i][RCOMP], 255 - arg2[i][ACOMP])) >> RGBshift);
-               rgba[i][GCOMP] = (GLubyte) ((PROD(arg0[i][GCOMP], arg2[i][ACOMP]) + PROD(arg1[i][GCOMP], 255 - arg2[i][ACOMP])) >> RGBshift);
-               rgba[i][BCOMP] = (GLubyte) ((PROD(arg0[i][BCOMP], arg2[i][ACOMP]) + PROD(arg1[i][BCOMP], 255 - arg2[i][ACOMP])) >> RGBshift);
+               GLuint r = (PROD(arg0[i][RCOMP], arg2[i][RCOMP])
+                           + PROD(arg1[i][RCOMP], 255 - arg2[i][RCOMP]))
+                              >> RGBshift;
+               GLuint g = (PROD(arg0[i][GCOMP], arg2[i][GCOMP])
+                           + PROD(arg1[i][GCOMP], 255 - arg2[i][GCOMP]))
+                              >> RGBshift;
+               GLuint b = (PROD(arg0[i][BCOMP], arg2[i][BCOMP])
+                           + PROD(arg1[i][BCOMP], 255 - arg2[i][BCOMP]))
+                              >> RGBshift;
+               rgba[i][RCOMP] = (GLubyte) MIN2(r, 255);
+               rgba[i][GCOMP] = (GLubyte) MIN2(g, 255);
+               rgba[i][BCOMP] = (GLubyte) MIN2(b, 255);
+            }
+         }
+	 break;
+      case GL_DOT3_RGB_EXT:
+      case GL_DOT3_RGBA_EXT:
+         {
+            const GLubyte (*arg0)[4] = (const GLubyte (*)[4]) argRGB[0];
+            const GLubyte (*arg1)[4] = (const GLubyte (*)[4]) argRGB[1];
+	    /* ATI's EXT extension has a constant scale by 4.  The ARB
+	     * one will likely remove this restriction, and we should
+	     * drop the EXT extension in favour of the ARB one.
+	     */
+            RGBshift = 6;
+            for (i = 0; i < n; i++) {
+               GLint dot = (S_PROD((GLint) arg0[i][RCOMP] - 128,
+                                   (GLint) arg1[i][RCOMP] - 128) +
+                            S_PROD((GLint) arg0[i][GCOMP] - 128,
+                                   (GLint) arg1[i][GCOMP] - 128) +
+                            S_PROD((GLint) arg0[i][BCOMP] - 128,
+                                   (GLint) arg1[i][BCOMP] - 128)) >> RGBshift;
+               rgba[i][RCOMP] = (GLubyte) CLAMP(dot, 0, 255);
+               rgba[i][GCOMP] = (GLubyte) CLAMP(dot, 0, 255);
+               rgba[i][BCOMP] = (GLubyte) CLAMP(dot, 0, 255);
             }
          }
          break;
@@ -2202,8 +2250,17 @@ _mesa_texture_combine(CONST GLcontext *ctx,
       case GL_REPLACE:
          {
             const GLubyte (*arg0)[4] = (const GLubyte (*)[4]) argA[0];
-            for (i = 0; i < n; i++)
-               rgba[i][ACOMP] = arg0[i][ACOMP] << Ashift;
+            if (Ashift) {
+               for (i = 0; i < n; i++) {
+                  GLuint a = (GLuint) arg0[i][ACOMP] << Ashift;
+                  rgba[i][ACOMP] = (GLubyte) MIN2(a, 255);
+               }
+            }
+            else {
+               for (i = 0; i < n; i++) {
+                  rgba[i][ACOMP] = arg0[i][ACOMP];
+               }
+            }
          }
          break;
       case GL_MODULATE:
@@ -2211,8 +2268,10 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             const GLubyte (*arg0)[4] = (const GLubyte (*)[4]) argA[0];
             const GLubyte (*arg1)[4] = (const GLubyte (*)[4]) argA[1];
             Ashift = 8 - Ashift;
-            for (i = 0; i < n; i++)
-               rgba[i][ACOMP] = (GLubyte) (PROD(arg0[i][ACOMP], arg1[i][ACOMP]) >> Ashift);
+            for (i = 0; i < n; i++) {
+               GLuint a = (PROD(arg0[i][ACOMP], arg1[i][ACOMP]) >> Ashift);
+               rgba[i][ACOMP] = (GLubyte) MIN2(a, 255);
+            }
          }
          break;
       case GL_ADD:
@@ -2221,7 +2280,7 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             const GLubyte  (*arg1)[4] = (const GLubyte (*)[4]) argA[1];
             for (i = 0; i < n; i++) {
                GLint a = ((GLint) arg0[i][ACOMP] + arg1[i][ACOMP]) << Ashift;
-               rgba[i][ACOMP] = (a > 255) ? 255 : (GLubyte) a;
+               rgba[i][ACOMP] = (GLubyte) MIN2(a, 255);
             }
          }
          break;
@@ -2232,7 +2291,7 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             for (i = 0; i < n; i++) {
                GLint a = (GLint) arg0[i][ACOMP] + (GLint) arg1[i][ACOMP] - 128;
                a = (a < 0) ? 0 : a << Ashift;
-               rgba[i][ACOMP] = (a > 255) ? 255 : (GLubyte) a;
+               rgba[i][ACOMP] = (GLubyte) MIN2(a, 255);
             }
          }
          break;
@@ -2242,13 +2301,25 @@ _mesa_texture_combine(CONST GLcontext *ctx,
             const GLubyte (*arg1)[4] = (const GLubyte (*)[4]) argA[1];
             const GLubyte (*arg2)[4] = (const GLubyte (*)[4]) argA[2];
             Ashift = 8 - Ashift;
-            for (i=0; i<n; i++)
-               rgba[i][ACOMP] = (GLubyte) ((PROD(arg0[i][ACOMP], arg2[i][ACOMP]) + PROD(arg1[i][ACOMP], 255 - arg2[i][ACOMP])) >> Ashift);
+            for (i=0; i<n; i++) {
+               GLuint a = (PROD(arg0[i][ACOMP], arg2[i][ACOMP])
+                           + PROD(arg1[i][ACOMP], 255 - arg2[i][ACOMP]))
+                              >> Ashift;
+               rgba[i][ACOMP] = (GLubyte) MIN2(a, 255);
+            }
          }
          break;
       default:
          gl_problem(NULL, "invalid combine mode");
    }
+
+   /* Fix the alpha component for GL_DOT3_RGBA_EXT combining.
+    */
+   if (textureUnit->CombineModeRGB == GL_DOT3_RGBA_EXT) {
+      for (i = 0; i < n; i++) {
+	 rgba[i][ACOMP] = rgba[i][RCOMP];
+      }
+   }
 }
 #undef PROD
 
@@ -2270,7 +2341,7 @@ _mesa_texture_combine(CONST GLcontext *ctx,
  *                according to the texture environment mode.
  */
 static void apply_texture( CONST GLcontext *ctx,
-                           CONST struct gl_texture_unit *texUnit,
+                           const struct gl_texture_unit *texUnit,
                            GLuint n,
                            GLubyte primary_rgba[][4], GLubyte texel[][4],
                            GLubyte rgba[][4] )
@@ -2646,7 +2717,7 @@ static void apply_texture( CONST GLcontext *ctx,
                gl_problem(ctx, "Bad format in apply_texture (GL_COMBINE_EXT)");
                return;
          }
-         _mesa_texture_combine (ctx, texUnit, n, primary_rgba, texel, rgba);
+         _mesa_texture_combine(ctx, texUnit, n, primary_rgba, texel, rgba);
          break;
 
       default:
@@ -2667,7 +2738,7 @@ void gl_texture_pixels( GLcontext *ctx, GLuint texUnit, GLuint n,
                         GLubyte primary_rgba[][4], GLubyte rgba[][4] )
 {
    GLuint mask = (TEXTURE0_1D | TEXTURE0_2D | TEXTURE0_3D | TEXTURE0_CUBE) << (texUnit * 4);
-   if (ctx->Texture.Enabled & mask) {
+   if (ctx->Texture.ReallyEnabled & mask) {
       const struct gl_texture_unit *textureUnit = &ctx->Texture.Unit[texUnit];
       if (textureUnit->Current && textureUnit->Current->SampleFunc) {
          GLubyte texel[PB_SIZE][4];
@@ -2692,41 +2763,9 @@ void gl_texture_pixels( GLcontext *ctx, GLuint texUnit, GLuint n,
          }
 
          /* fetch texture images from device driver, if needed */
-         {
-            static const GLenum targets[] = {
-               GL_TEXTURE_1D,
-               GL_TEXTURE_2D,
-               GL_TEXTURE_3D,
-               GL_TEXTURE_CUBE_MAP_ARB,
-               GL_TEXTURE_CUBE_MAP_ARB,
-               GL_TEXTURE_CUBE_MAP_ARB
-            };
-            struct gl_texture_object *texObj = textureUnit->Current;
-            GLboolean needLambda = (texObj->MinFilter != texObj->MagFilter);
-            GLenum target = targets[texObj->Dimensions - 1];
-            if (needLambda) {
-               GLint level;
-               /* Get images for all mipmap levels.  We might not need them
-                * all but this is easier.  We're on a (slow) software path
-                * anyway.
-                */
-               for (level = 0; level <= texObj->P; level++) {
-                  struct gl_texture_image *texImg = texObj->Image[level];
-                  if (texImg && !texImg->Data) {
-                     _mesa_get_teximage_from_driver(ctx, target, level, texObj);
-                     if (!texImg->Data)
-                        return;  /* out of memory */
-                  }
-               }
-            }
-            else {
-               GLint level = texObj->BaseLevel;
-               struct gl_texture_image *texImg = texObj->Image[level];
-               if (texImg && !texImg->Data) {
-                  _mesa_get_teximage_from_driver(ctx, target, level, texObj);
-                  if (!texImg->Data)
-                     return;  /* out of memory */
-               }
+         if (ctx->Driver.GetTexImage) {
+            if (!_mesa_get_teximages_from_driver(ctx, textureUnit->Current)) {
+               return;
             }
          }
 
