@@ -1,22 +1,21 @@
 /* -*- mode: C; tab-width:8; c-basic-offset:8 -*- */
-/* $Id: types.h,v 1.70 2000/07/12 12:16:06 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
- * 
+ * Version:  3.4
+ *
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
@@ -28,7 +27,7 @@
 
 #ifndef TYPES_H
 #define TYPES_H
- 
+
 
 #include "glheader.h"
 #include "config.h"
@@ -127,7 +126,7 @@ typedef void (*rect_func)( GLcontext *ctx, GLint x, GLint y,
 
 
 /*
- * The auxiliary interpolation function for clipping - now always in 
+ * The auxiliary interpolation function for clipping - now always in
  * clip space.
  */
 typedef void (*clip_interp_func)( struct vertex_buffer *VB, GLuint dst,
@@ -141,7 +140,7 @@ typedef void (*render_vb_func)( GLcontext *ctx );
 
 
 
-typedef void (*render_func)( struct vertex_buffer *VB, 
+typedef void (*render_func)( struct vertex_buffer *VB,
 			     GLuint start,
 			     GLuint count,
 			     GLuint parity );
@@ -288,7 +287,7 @@ struct gl_light {
    GLfloat VP_inf_norm[3];	/* Norm direction to infinite light */
    GLfloat h_inf_norm[3];	/* Norm( VP_inf_norm + <0,0,1> ) */
    GLfloat NormDirection[4];	/* normalized spotlight direction */
-   GLfloat VP_inf_spot_attenuation; 
+   GLfloat VP_inf_spot_attenuation;
 
    GLfloat SpotExpTable[EXP_TABLE_SIZE][2];  /* to replace a pow() call */
    GLfloat MatAmbient[2][3];	/* material ambient * light ambient */
@@ -369,7 +368,7 @@ struct gl_colorbuffer_attrib {
 
 
 struct gl_current_attrib {
-   /* KW: These values valid only when the VB is flushed.  
+   /* KW: These values valid only when the VB is flushed.
     */
    GLuint Flag;		                	/* Contains size information */
    GLfloat Normal[3];
@@ -378,7 +377,7 @@ struct gl_current_attrib {
    GLboolean EdgeFlag;				/* Current edge flag */
    GLfloat Texcoord[MAX_TEXTURE_UNITS][4];	/* Current texture coords */
    GLenum Primitive;		                /* Prim or GL_POLYGON+1 */
-	
+
    /* KW: No change to these values.
     */
    GLfloat RasterPos[4];			/* Current raster position */
@@ -450,7 +449,7 @@ struct gl_enable_attrib {
    GLboolean RescaleNormals;
    GLboolean Scissor;
    GLboolean Stencil;
-   GLuint Texture;
+   GLuint Texture[MAX_TEXTURE_UNITS];
    GLuint TexGen[MAX_TEXTURE_UNITS];
 };
 
@@ -666,11 +665,11 @@ struct gl_point_attrib {
    GLboolean SmoothFlag;/* True if GL_POINT_SMOOTH is enabled */
    GLfloat UserSize;	/* User-specified point size */
    GLfloat Size;	/* Point size actually used */
-   GLfloat Params[3];	/* Point Parameters EXT distance atenuation 
-			   factors by default = {1,0,0} */ 
+   GLfloat Params[3];	/* Point Parameters EXT distance atenuation
+			   factors by default = {1,0,0} */
    GLfloat MinSize;	/* Default 0.0 always >=0 */
    GLfloat MaxSize;	/* Default MAX_POINT_SIZE */
-   GLfloat Threshold;	/* Default 1.0 */ 
+   GLfloat Threshold;	/* Default 1.0 */
    GLboolean Attenuated;
 };
 
@@ -776,7 +775,7 @@ struct gl_stencil_attrib {
 #define ENABLE_TEX_ANY  (ENABLE_TEX0 | ENABLE_TEX1)
 
 
-typedef void (*texgen_func)( struct vertex_buffer *VB, 
+typedef void (*texgen_func)( struct vertex_buffer *VB,
 			     GLuint textureSet);
 
 
@@ -829,8 +828,8 @@ struct gl_texture_object {
  * Texture units are new with the multitexture extension.
  */
 struct gl_texture_unit {
-   GLuint Enabled;
-   GLuint ReallyEnabled;
+   GLuint Enabled;              /* bitmask of TEXTURE0_1D, _2D, _3D, _CUBE */
+   GLuint ReallyEnabled;        /* 0 or one of TEXTURE0_1D, _2D, _3D, _CUBE */
 
    GLenum EnvMode;	   /* GL_MODULATE, GL_DECAL, GL_BLEND, GL_COMBINE_EXT */
    GLenum LastEnvMode;
@@ -887,17 +886,15 @@ struct gl_texture_attrib {
    GLuint CurrentUnit;			/* Current texture unit */
    GLuint CurrentTransformUnit;		/* Current texture xform unit */
 
-   /* Bitwise-OR of TEXTURE_XD values */
-   GLuint Enabled;		/* Enabled by the user */
-   GLuint ReallyEnabled;	/* Really enabled (w.r.t. completeness, etc) */
-	
+   GLuint ReallyEnabled;     /* Really enabled (w.r.t. completeness, etc) */
+
    GLuint LastEnabled;	/* Decide whether enabled has really changed */
 
    GLboolean NeedNormals;
    GLboolean NeedEyeCoords;
 
    struct gl_texture_unit Unit[MAX_TEXTURE_UNITS];
-	
+
    struct gl_texture_object *Proxy1D;
    struct gl_texture_object *Proxy2D;
    struct gl_texture_object *Proxy3D;
@@ -974,7 +971,7 @@ struct gl_client_array {
  */
 typedef void (*trans_1ui_func)(GLuint *to,
 			       const struct gl_client_array *from,
-			       GLuint start, 
+			       GLuint start,
 			       GLuint n );
 
 typedef void (*trans_1ub_func)(GLubyte *to,
@@ -989,59 +986,63 @@ typedef void (*trans_4ub_func)(GLubyte (*to)[4],
 
 typedef void (*trans_4f_func)(GLfloat (*to)[4],
 			      const struct gl_client_array *from,
-			      GLuint start, 
+			      GLuint start,
 			      GLuint n );
 
 typedef void (*trans_3f_func)(GLfloat (*to)[3],
 			      const struct gl_client_array *from,
-			      GLuint start, 
+			      GLuint start,
 			      GLuint n );
 
 
 
 typedef void (*trans_elt_1ui_func)(GLuint *to,
 				   const struct gl_client_array *from,
-				   GLuint *flags, 
-				   GLuint *elts, 
+				   GLuint *flags,
+				   GLuint *elts,
 				   GLuint match,
 				   GLuint start,
 				   GLuint n );
 
 typedef void (*trans_elt_1ub_func)(GLubyte *to,
 				   const struct gl_client_array *from,
-				   GLuint *flags, 
-				   GLuint *elts, 
+				   GLuint *flags,
+				   GLuint *elts,
 				   GLuint match,
 				   GLuint start,
 				   GLuint n );
 
 typedef void (*trans_elt_4ub_func)(GLubyte (*to)[4],
 				   const struct gl_client_array *from,
-				   GLuint *flags, 
-				   GLuint *elts, 
+				   GLuint *flags,
+				   GLuint *elts,
 				   GLuint match,
 				   GLuint start,
 				   GLuint n );
 
 typedef void (*trans_elt_4f_func)(GLfloat (*to)[4],
 				  const struct gl_client_array *from,
-				  GLuint *flags, 
-				  GLuint *elts, 
+				  GLuint *flags,
+				  GLuint *elts,
 				  GLuint match,
 				  GLuint start,
 				  GLuint n );
 
 typedef void (*trans_elt_3f_func)(GLfloat (*to)[3],
 				  const struct gl_client_array *from,
-				  GLuint *flags, 
-				  GLuint *elts, 
+				  GLuint *flags,
+				  GLuint *elts,
 				  GLuint match,
 				  GLuint start,
 				  GLuint n );
 
 
+/*#define VAO*/
+#ifdef VAO
+struct gl_array_object {
+   GLuint Name;
+   GLuint RefCount;
 
-struct gl_array_attrib {
    struct gl_client_array Vertex;	     /* client data descriptors */
    struct gl_client_array Normal;
    struct gl_client_array Color;
@@ -1050,15 +1051,54 @@ struct gl_array_attrib {
    struct gl_client_array EdgeFlag;
 
    trans_4f_func  VertexFunc;       /* conversion functions */
-   trans_3f_func  NormalFunc;  
-   trans_4ub_func ColorFunc;   
+   trans_3f_func  NormalFunc;
+   trans_4ub_func ColorFunc;
    trans_1ui_func IndexFunc;
    trans_4f_func  TexCoordFunc[MAX_TEXTURE_UNITS];
    trans_1ub_func EdgeFlagFunc;
 
    trans_elt_4f_func  VertexEltFunc; /* array elt conversion functions */
-   trans_elt_3f_func  NormalEltFunc;  
-   trans_elt_4ub_func ColorEltFunc;   
+   trans_elt_3f_func  NormalEltFunc;
+   trans_elt_4ub_func ColorEltFunc;
+   trans_elt_1ui_func IndexEltFunc;
+   trans_elt_4f_func  TexCoordEltFunc[MAX_TEXTURE_UNITS];
+   trans_elt_1ub_func EdgeFlagEltFunc;
+
+   GLint TexCoordInterleaveFactor;
+
+   GLuint LockFirst;
+   GLuint LockCount;
+
+   GLuint Flag[VB_SIZE];	/* crock */
+   GLuint Flags;
+   GLuint Summary;		/* Like flags, but no size information */
+};
+#endif
+
+
+struct gl_array_attrib {
+#ifdef VAO
+   struct gl_array_object *Current;
+   GLint ActiveTexture;		/* Client Active Texture */
+   GLuint NewArrayState;	/* Tracks which arrays have been changed. */
+#else
+   struct gl_client_array Vertex;	     /* client data descriptors */
+   struct gl_client_array Normal;
+   struct gl_client_array Color;
+   struct gl_client_array Index;
+   struct gl_client_array TexCoord[MAX_TEXTURE_UNITS];
+   struct gl_client_array EdgeFlag;
+
+   trans_4f_func  VertexFunc;       /* conversion functions */
+   trans_3f_func  NormalFunc;
+   trans_4ub_func ColorFunc;
+   trans_1ui_func IndexFunc;
+   trans_4f_func  TexCoordFunc[MAX_TEXTURE_UNITS];
+   trans_1ub_func EdgeFlagFunc;
+
+   trans_elt_4f_func  VertexEltFunc; /* array elt conversion functions */
+   trans_elt_3f_func  NormalEltFunc;
+   trans_elt_4ub_func ColorEltFunc;
    trans_elt_1ui_func IndexEltFunc;
    trans_elt_4f_func  TexCoordEltFunc[MAX_TEXTURE_UNITS];
    trans_elt_1ub_func EdgeFlagEltFunc;
@@ -1068,11 +1108,12 @@ struct gl_array_attrib {
 
    GLuint LockFirst;
    GLuint LockCount;
-	
+
    GLuint Flag[VB_SIZE];	/* crock */
    GLuint Flags;
    GLuint Summary;		/* Like flags, but no size information */
    GLuint NewArrayState;	/* Tracks which arrays have been changed. */
+#endif
 };
 
 
@@ -1159,7 +1200,7 @@ struct gl_pipeline {
 struct gl_cva {
    struct gl_pipeline pre;
    struct gl_pipeline elt;
-	
+
    struct gl_client_array Elt;
    trans_1ui_func EltFunc;
 
@@ -1213,7 +1254,7 @@ struct gl_1d_map {
    GLfloat u1, u2, du;	/* u1, u2, 1.0/(u2-u1) */
    GLfloat *Points;	/* Points to contiguous control points */
 };
-	
+
 
 /*
  * 2-D Evaluator control points
@@ -1264,12 +1305,17 @@ struct gl_shared_state {
    GLint RefCount;			   /* Reference count */
    struct _mesa_HashTable *DisplayList;	   /* Display lists hash table */
    struct _mesa_HashTable *TexObjects;	   /* Texture objects hash table */
+#ifdef VAO
+   struct _mesa_HashTable *ArrayObjects;   /* GL_EXT_vertex_array_set */
+#endif
    struct gl_texture_object *TexObjectList;/* Linked list of texture objects */
    struct gl_texture_object *DirtyTexObjList; /* List of dirty tex objects */
 
    /* Default texture objects (shared by all multi-texture units) */
    struct gl_texture_object *DefaultD[4];
    struct gl_texture_object *DefaultCubeMap;
+
+   void *DriverData;  /* Device driver shared state */
 };
 
 
@@ -1377,6 +1423,7 @@ struct gl_extensions {
    /* flags to quickly test if certain extensions are available */
    GLboolean HaveTextureEnvAdd;
    GLboolean HaveTextureEnvCombine;
+   GLboolean HaveTextureEnvDot3;
    GLboolean HaveTextureLodBias;
    GLboolean HaveHpOcclusionTest;
    GLboolean HaveTextureCubeMap;
@@ -1432,7 +1479,7 @@ struct gl_extensions {
 
 
 #define NEW_DRIVER_STATE (NEW_DRVSTATE0 | NEW_DRVSTATE1 |	\
-                          NEW_DRVSTATE2 | NEW_DRVSTATE3) 
+                          NEW_DRVSTATE2 | NEW_DRVSTATE3)
 
 /* What can the driver do, what requires us to call render_triangle or
  * a non-driver rasterize function?
@@ -1455,13 +1502,13 @@ struct gl_extensions {
 #define DD_POINT_SIZE               0x8000
 #define DD_POINT_ATTEN              0x10000
 #define DD_LIGHTING_CULL            0x20000 /* never set by driver */
-#define DD_POINT_SW_RASTERIZE       0x40000 
-#define DD_LINE_SW_RASTERIZE        0x80000 
-#define DD_TRI_SW_RASTERIZE         0x100000 
-#define DD_QUAD_SW_RASTERIZE        0x200000 
+#define DD_POINT_SW_RASTERIZE       0x40000
+#define DD_LINE_SW_RASTERIZE        0x80000
+#define DD_TRI_SW_RASTERIZE         0x100000
+#define DD_QUAD_SW_RASTERIZE        0x200000
 #define DD_TRI_CULL_FRONT_BACK      0x400000 /* not supported by most drivers */
-#define DD_Z_NEVER                  0x800000 
-#define DD_STENCIL                  0x1000000 
+#define DD_Z_NEVER                  0x800000
+#define DD_STENCIL                  0x1000000
 #define DD_CLIP_FOG_COORD           0x2000000
 
 
@@ -1482,7 +1529,7 @@ struct gl_extensions {
                                DD_QUAD_SW_RASTERIZE)
 
 
-/* Vertex buffer clipping flags 
+/* Vertex buffer clipping flags
  */
 #define CLIP_RIGHT_SHIFT 	0
 #define CLIP_LEFT_SHIFT 	1
@@ -1534,7 +1581,7 @@ struct gl_extensions {
  */
 #define SHADE_TWOSIDE           0x4
 
-/* Flags for selecting a normal transformation function. 
+/* Flags for selecting a normal transformation function.
  */
 #define NORM_RESCALE   0x1	/* apply the scale factor */
 #define NORM_NORMALIZE 0x2	/* normalize */
@@ -1630,7 +1677,7 @@ typedef union node Node;
 
 
 /* KW: Flags that describe the current vertex state, and the contents
- * of a vertex in a vertex-cassette.  
+ * of a vertex in a vertex-cassette.
  *
  * For really major expansion, consider a 'VERT_ADDITIONAL_FLAGS' flag,
  * which means there is data in another flags array (eg, extra_flags[]).
@@ -1647,7 +1694,7 @@ typedef union node Node;
 #define VERT_INDEX           0x100	/* glIndex */
 #define VERT_EDGE            0x200	/* glEdgeFlag */
 #define VERT_MATERIAL        0x400	/* glMaterial */
-#define VERT_TEX0_1          0x800	   
+#define VERT_TEX0_1          0x800
 #define VERT_TEX0_2          0x1000
 #define VERT_TEX0_3          0x2000
 #define VERT_TEX0_4          0x4000
@@ -1664,7 +1711,7 @@ typedef union node Node;
 #define VERT_EVAL_C2         0x2000000  /*    - or just use 3 bits */
 #define VERT_EVAL_P1         0x4000000  /*  */
 #define VERT_EVAL_P2         0x8000000  /*  */
-#define VERT_SPEC_RGB        0x10000000	
+#define VERT_SPEC_RGB        0x10000000
 #define VERT_FOG_COORD       0x20000000	/* internal use only, currently */
 
 #define VERT_EYE             VERT_BEGIN /* for pipeline management & cva */
@@ -1700,7 +1747,7 @@ typedef union node Node;
 #define VERT_TEX2_1234    (VERT_TEX2_4|VERT_TEX2_123)
 #define VERT_TEX2_ANY     VERT_TEX2_1
 
-/* not used 
+/* not used
 #define VERT_TEX3_12      (VERT_TEX3_2|VERT_TEX3_1)
 #define VERT_TEX3_123     (VERT_TEX3_3|VERT_TEX3_12)
 #define VERT_TEX3_1234    (VERT_TEX3_4|VERT_TEX3_123)
@@ -1739,7 +1786,7 @@ typedef union node Node;
 typedef void (*vb_transform_func)( GLcontext *ctx );
 
 
-typedef GLuint (*clip_line_func)( struct vertex_buffer *VB, 
+typedef GLuint (*clip_line_func)( struct vertex_buffer *VB,
 				  GLuint *i, GLuint *j,
 				  GLubyte mask);
 typedef GLuint (*clip_poly_func)( struct vertex_buffer *VB,
@@ -1747,7 +1794,7 @@ typedef GLuint (*clip_poly_func)( struct vertex_buffer *VB,
 				  GLubyte mask );
 
 /*
- * The library context: 
+ * The library context:
  */
 
 struct gl_context {
@@ -1765,7 +1812,7 @@ struct gl_context {
 
    /* Driver function pointer table */
    struct dd_function_table Driver;
-	
+
    triangle_func TriangleFunc; /* driver or indirect triangle func */
    quad_func     QuadFunc;
    triangle_func ClippedTriangleFunc;
@@ -1885,7 +1932,7 @@ struct gl_context {
    GLboolean IntegerAccumMode;	/* Storing unscaled integers? */
    GLfloat IntegerAccumScaler;	/* Implicit scale factor */
 
-   struct gl_fallback_arrays Fallback; 
+   struct gl_fallback_arrays Fallback;
 
    GLenum ErrorValue;        /* Last error code */
 
@@ -1913,7 +1960,7 @@ struct gl_context {
    clip_interp_func ClipInterpFunc;
    GLuint ClipTabMask;
 
-   normal_func *NormalTransform; 
+   normal_func *NormalTransform;
 
    /* Current shading function table */
    gl_shade_func *shade_func_tab;
@@ -1946,7 +1993,7 @@ struct gl_context {
    /* The vertex buffer being used by this context.
     */
    struct vertex_buffer *VB;
-   
+
    /* The pixel buffer being used by this context */
    struct pixel_buffer* PB;
 
@@ -1955,18 +2002,22 @@ struct gl_context {
 
    /* Should 3Dfx Glide driver catch signals? */
    GLboolean CatchSignals;
-        
+
    /* For debugging/development only */
    GLboolean NoRaster;
    GLboolean FirstTimeCurrent;
 
    /* Dither disable via MESA_NO_DITHER env var */
    GLboolean NoDither;
+
+#ifdef DEBUG
+   GLboolean Rendering;
+#endif
 };
 
 
 #ifdef MESA_DEBUG
-extern int MESA_VERBOSE; 
+extern int MESA_VERBOSE;
 extern int MESA_DEBUG_FLAGS;
 #else
 # define MESA_VERBOSE 0
@@ -1989,12 +2040,12 @@ enum _verbose {
 	VERBOSE_CULL            = 0x100,
 	VERBOSE_DISPLAY_LIST    = 0x200,
 	VERBOSE_LIGHTING        = 0x400
-}; 
+};
 
 
 enum _debug {
 	DEBUG_ALWAYS_FLUSH          = 0x1
-}; 
+};
 
 
 extern void gl_flush_vb( GLcontext *ctx, const char *where );
@@ -2011,7 +2062,7 @@ do {						\
 
 
 
-/* Test if we're inside a glBegin / glEnd pair: 
+/* Test if we're inside a glBegin / glEnd pair:
 */
 #define ASSERT_OUTSIDE_BEGIN_END( ctx, where )				\
 do {									\
@@ -2070,7 +2121,44 @@ do {								\
 
 #define Elements(x) sizeof(x)/sizeof(*(x))
 
+
+#ifdef DEBUG
+
+#define RENDER_START(CTX)			\
+   do {						\
+      assert(!(CTX)->Rendering);		\
+      (CTX)->Rendering = GL_TRUE;		\
+      if ((CTX)->Driver.RenderStart) {		\
+         (*(CTX)->Driver.RenderStart)(CTX);	\
+      }						\
+   } while (0)
+
+#define RENDER_FINISH(CTX)			\
+   do {						\
+      assert((CTX)->Rendering);			\
+      (CTX)->Rendering = GL_FALSE;		\
+      if ((CTX)->Driver.RenderFinish) {		\
+         (*(CTX)->Driver.RenderFinish)(CTX);	\
+      }						\
+   } while (0)
+
+#else
+
+#define RENDER_START(CTX)			\
+   do {						\
+      if ((CTX)->Driver.RenderStart) {		\
+         (*(CTX)->Driver.RenderStart)(CTX);	\
+      }						\
+   } while (0)
+
+#define RENDER_FINISH(CTX)			\
+   do {						\
+      if ((CTX)->Driver.RenderFinish) {		\
+         (*(CTX)->Driver.RenderFinish)(CTX);	\
+      }						\
+   } while (0)
+
 #endif
 
 
-
+#endif /* TYPES_H */
